#!/usr/bin/env python
# Licensed under a 3-clause BSD style license, see LICENSE.
"""
Tests for the hepunits.constants.constants module.
"""

from pytest import approx

from hepunits import *
from hepunits import THz, eV, nanometer, s


def test_constants():
    assert pi_sq == two_pi * half_pi
    assert Avogadro == 6.02214076e23
    assert c_light / (m / s) == 299792458
    assert hbarc_sq / c_light_sq == approx((h_Planck / two_pi) ** 2)
    assert hbar / (eV * s) == hbar / 1.0e3
    # wavelength of 555-ish nanometres (green light) has a frequency of 540 THz
    assert c_light / (555.17121851852 * nanometer) == approx(540 * THz)



# Licensed under a 3-clause BSD style license, see LICENSE.

from . import constants
from .constants import (
    Avogadro,
    c_light,
    c_light_sq,
    e_SI,
    e_sq,
    electronvolt,
    eminus,
    eplus,
    h_Planck,
    half_pi,
    hbar,
    hbar_Planck,
    hbarc,
    hbarc_sq,
    joule,
    k_Boltzmann,
    kelvin,
    m,
    mole,
    pi,
    pi_sq,
    s,
    two_pi,
)

__all__ = (
    "Avogadro",
    "c_light",
    "c_light_sq",
    "constants",
    "e_SI",
    "e_sq",
    "electronvolt",
    "eminus",
    "eplus",
    "h_Planck",
    "half_pi",
    "hbar",
    "hbar_Planck",
    "hbarc",
    "hbarc_sq",
    "joule",
    "k_Boltzmann",
    "kelvin",
    "m",
    "mole",
    "pi",
    "pi_sq",
    "s",
    "two_pi",
)


def __dir__() -> list[str]:
    return list(__all__)



#!/usr/bin/env python
# Licensed under a 3-clause BSD style license, see LICENSE.
"""
Tests for the hepunits.units.units module.
"""

from pytest import approx

from hepunits import *
from hepunits import two_pi


def test_length():
    assert 3 * mm == approx(m * 0.003)
    assert 3 * mm == approx(cm * 0.3)
    assert 3 * mm == approx(km * 0.000003)
    assert 3 * mm == approx(micrometer * 3000)
    assert 3 * mm == approx(nanometer * 3000000)
    assert 3 * mm == approx(angstrom * 30000000)
    assert 3 * mm == approx(fermi * 3e12)


def test_area():
    assert (3 * mm) ** 2 == approx(9 * mm2)
    assert (3 * cm) ** 2 == approx(9 * cm2)
    assert (3 * km) ** 2 == approx(9 * km2)
    assert 2e-28 * meter2 == approx(2 * barn)


def test_crosssection():
    assert 3e-31 * barn == approx(9 * millibarn)
    assert 3e-34 * barn == approx(9 * microbarn)
    assert 3e-37 * barn == approx(9 * nanobarn)
    assert 3e-40 * barn == approx(9 * picobarn)
    assert 1 * barn == approx(100 * fm2)


def test_luminosity():
    assert invpb / invfb == approx(1.0e-3)


def test_volume():
    assert (3 * mm) ** 3 == approx(27 * mm3)
    assert (3 * cm) ** 3 == approx(27 * cm3)
    assert (3 * m) ** 3 == approx(27 * m3)


def test_time():
    assert 3 * ns == approx(3e-9 * s)
    assert 3 * ns == approx(3e-6 * ms)
    assert 3 * ns == approx(3e-3 * microsecond)
    assert 3 * ns == approx(3000 * picosecond)
    assert 3 * ns == approx(3000000 * femtosecond)
    assert day == 24 * 60 * minute


def test_frequency():
    assert second**-1 == approx(Hz)
    assert 1000 * hertz == approx(kHz)
    assert 1000000 * hertz == approx(MHz)
    assert 1000000000 * hertz == approx(GHz)


def test_energy():
    assert 1e3 * eV == approx(keV)
    assert 1e6 * eV == approx(MeV)
    assert 1e9 * eV == approx(GeV)
    assert 1e12 * eV == approx(TeV)
    assert 1e15 * eV == approx(PeV)
    assert 1e18 * eV == approx(EeV)


def test_angle():
    assert 360.0 * degree == two_pi * radian


def test_magnetic_field():
    assert 10 * gauss == approx(1 * milli * tesla)


def test_electricity():
    assert 1 * mega * joule / second == approx(1 * MW)


def test_radiation_units():
    assert gray == sievert  # equal in terms of value
    assert 1 * curie == 37 * giga * becquerel


def test_magnetic_flux():
    assert weber == tesla * meter2
    assert weber == ohm * coulomb



#!/usr/bin/env python
# Licensed under a 3-clause BSD style license, see LICENSE.
"""
Tests for the hepunits.pint module.
"""

import pint
import pytest
from pytest import approx

import hepunits
from hepunits.pint import from_clhep, to_clhep


def test_pint_constants():
    ureg = pint.UnitRegistry()

    # These three constants set the relationship between mass, length, time, and charge
    # TODO: check all 7 SI defining constants (hence also supporting more unit conversions)
    pint_c = to_clhep(1 * ureg.speed_of_light)
    assert pint_c == approx(hepunits.c_light, rel=1e-15)
    pint_h = to_clhep(1 * ureg.planck_constant)
    assert pint_h == approx(hepunits.h_Planck, rel=1e-15)
    pint_e = to_clhep(1 * ureg.elementary_charge)
    assert pint_e == approx(1.0, rel=1e-15)


def test_pint_roundtrip():
    ureg = pint.UnitRegistry()

    assert to_clhep(3 * ureg.mm) == approx(3.0)
    assert to_clhep(3 * ureg.cm) == approx(30.0)
    assert to_clhep(2 * ureg.ohm) == approx(2.0 * hepunits.ohm)
    assert to_clhep(ureg.coulomb) == approx(hepunits.coulomb)

    assert from_clhep(hepunits.c_light, ureg.meter / ureg.second).m == approx(
        (1.0 * ureg.c).to(ureg.meter / ureg.second).m, rel=1e-15
    )
    assert from_clhep(hepunits.tesla, ureg.tesla).m == approx(
        (1 * ureg.tesla).m, rel=1e-15
    )


def test_unsupported_dimension():
    ureg = pint.UnitRegistry()
    with pytest.raises(ValueError, match="Unsupported dimension"):
        to_clhep(1 * ureg.kelvin)


def test_consistent_registry():
    ureg = pint.UnitRegistry()

    a = 1 * hepunits.mm
    b = 3 * ureg.nanosecond
    assert a * to_clhep(b) == 3 * hepunits.mm * hepunits.nanosecond
    assert from_clhep(a, ureg.mm) * b == 3 * ureg.mm * ureg.nanosecond



import pytest

import hepunits


def filter_module(item: str) -> bool:
    if item == "__version__":
        return True
    if item.startswith("_"):
        return False
    return item not in {"List"}


@pytest.mark.parametrize(
    "module",
    [
        hepunits.units,
        hepunits.constants,
        hepunits.constants.constants,
        hepunits.units.prefixes,
        hepunits.units.units,
    ],
)
def test_missing_all(module):
    assert set(dir(module)) == set(module.__all__)

    full_module = {it for it in module.__dict__ if filter_module(it)}
    assert full_module == set(module.__all__)


def test_exported():
    expr = set(hepunits.units.__all__) - (set(hepunits.__all__) | {"prefixes"})
    assert not expr

    expr = set(hepunits.constants.__all__) - set(hepunits.__all__)
    assert not expr

    expr = set(hepunits.constants.constants.__all__) - set(hepunits.constants.__all__)
    assert not expr

    expr = set(hepunits.units.prefixes.__all__) - set(hepunits.units.__all__)
    assert not expr

    expr = set(hepunits.units.units.__all__) - set(hepunits.units.__all__)
    assert not expr
